---
# Documentation: https://sourcethemes.com/academic/docs/managing-content/

title: "GPIO and Home Assistant"
date: 2021-02-28T17:41:07+01:00
lastmod: 2021-02-28T17:41:07+01:00
featured: false
draft: false

authors: ['michaelh']
projects: ['solarch']
---
As part of moving my home server to solar power, I need a way of switching over to mains if there's been a few grey days and the battery is getting low. I've attached a relay to the Raspberry Pi's GPIO to switch the charger input from solar to a [MEAN WELL 100 W supply](https://www.meanwell-web.com/en-gb/ac-dc-single-output-enclosed-power-supply-lrs--100--36) but integrating is surprisingly tricky:

- The [`rpi_gpio`](https://www.home-assistant.io/integrations/rpi_gpio/) module uses RPi.GPIO which isn't 64 bit clean and fails to compile on Ubuntu
- The [`remote_rpi_gpio`](https://www.home-assistant.io/integrations/remote_rpi_gpio/) module uses [pigpiod](http://abyz.me.uk/rpi/pigpio/pigpiod.html) which for some reason uses 20 % CPU when idle.

I settled on [`pi-mqtt-gpio`](https://github.com/flyte/pi-mqtt-gpio), as most of my other custom integrations are over MQTT. The config is:

```yaml
mqtt:
  host: localhost
  topic_prefix: host/keylime
  discovery: true

gpio_modules:
  - name: gpiod
    module: gpiod

digital_outputs:
  - name: mains_select
    module: gpiod
    pin: 17
    on_payload: "ON"
    off_payload: "OFF"
```

`discovery` is undocumented but enables Home Assistant [MQTT discovery](https://www.home-assistant.io/docs/mqtt/discovery/), so it's zero extra config.

This should be usable on non-Pi hosts as well due to using the gpiod module.
